@@ -10,7 +10,7 @@
    modify it under the terms of v2 of the GNU Lesser General Public
    License as published by the Free Software Foundation.
 
-   Flactag is distributed in the hope that it will be useful,
+   libmusicbrainz4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
@@ -18,10 +18,13 @@
    You should have received a copy of the GNU General Public License
    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
-     $Id: make-c-interface.cc 13259 2011-08-10 12:02:50Z adhawkins $
+     $Id$
 
 ----------------------------------------------------------------------------*/
 
+#include "config.h"
+#include "musicbrainz4/defines.h"
+
 #include <iostream>
 #include <fstream>
 #include <sstream>
@@ -312,16 +315,46 @@ void ProcessClass(const XMLNode& Node, std::ofstream& Source, std::ofstream& Inc
 
 					if ("string"==PropertyType)
 					{
+						bool Deprecated=false;
+						std::string Replacement;
+
+						if (ChildNode.isAttributeSet("deprecated"))
+						{
+							std::string StrDeprecated=ChildNode.getAttribute("deprecated");
+
+							if (StrDeprecated=="true")
+								Deprecated=true;
+
+							if (ChildNode.isAttributeSet("replacement"))
+								Replacement=ChildNode.getAttribute("replacement");
+						}
+
 						Include << "/**" << std::endl;
-						Include << " * @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << std::endl;
+						Include << " * ";
+						if (Deprecated)
+							Include << "@deprecated ";
+
+						Include << "@see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << std::endl;
 						Include << " *" << std::endl;
+
+						if (Deprecated)
+						{
+ 							Include << " * <b>This method is deprecated, please use #" << Replacement << "</b>" << std::endl;
+ 							Include << " *" << std::endl;
+ 						}
+
 						Include << " * @param " << UpperName << " #Mb4" << UpperName << " object" << std::endl;
 						Include << " * @param str Returned string" << std::endl;
 						Include << " * @param len Number of characters available in return string" << std::endl;
 						Include << " *" << std::endl;
 						Include << " * @return The number of characters in the string to copy (not including terminating NULL)" << std::endl;
 						Include << " */" << std::endl;
-						Include << "  int mb4_" << LowerName << "_get_" << PropertyLowerName << "(Mb4" << UpperName << " " << UpperName << ", char *str, int len);" << std::endl;
+						Include << "  ";
+
+						if (Deprecated)
+							Include << "LIBMB4_DEPRECATED(" << Replacement << ") ";
+
+						Include << "int mb4_" << LowerName << "_get_" << PropertyLowerName << "(Mb4" << UpperName << " " << UpperName << ", char *str, int len);" << std::endl;
 						Include << std::endl;
 
 						Source << "  MB4_C_STR_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "," << PropertyLowerName << ")" << std::endl;
@@ -368,6 +401,94 @@ void ProcessClass(const XMLNode& Node, std::ofstream& Source, std::ofstream& Inc
 
 						Source << "  MB4_C_OBJ_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "," << PropertyLowerName << ")" << std::endl;
 					}
+					else if ("relationlist"==PropertyType)
+					{
+						Include << "/**" << std::endl;
+						Include << " * @deprecated @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << std::endl;
+						Include << " *" << std::endl;
+						Include << " * <b>This method is deprecated, please use #mb4_" << LowerName << "_get_" << PropertyLowerName << "list</b>" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << " object" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return #Mb4" << PropertyUpperName << " object" << std::endl;
+						Include << " */" << std::endl;
+						Include << " LIBMB4_DEPRECATED(mb4_" << LowerName << "_get_" << PropertyLowerName << "list) Mb4" << PropertyUpperName << " mb4_" << LowerName << "_get_" << PropertyLowerName << "(Mb4" << UpperName << " " << UpperName << ");" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_OBJ_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "," << PropertyLowerName << ")" << std::endl;
+
+						Include << "/**" << std::endl;
+						Include << " * @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << "List" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << "List object" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return #Mb4" << PropertyUpperName << "List object" << std::endl;
+						Include << " */" << std::endl;
+						Include << "  Mb4" << PropertyUpperName << "List mb4_" << LowerName << "_get_" << PropertyLowerName << "list(Mb4" << UpperName << " " << UpperName << ");" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_OBJ_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "List," << PropertyLowerName << "list)" << std::endl;
+					}
+					else if ("iswc"==PropertyType)
+					{
+						Include << "/**" << std::endl;
+						Include << " * @deprecated @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << std::endl;
+						Include << " *" << std::endl;
+						Include << " * <b>This method is deprecated, please use #mb4_" << LowerName << "_get_" << PropertyLowerName << "list</b>" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << " object" << std::endl;
+						Include << " * @param str Returned string" << std::endl;
+						Include << " * @param len Number of characters available in return string" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return The number of characters in the string to copy (not including terminating NULL)" << std::endl;
+						Include << " */" << std::endl;
+						Include << "  LIBMB4_DEPRECATED(mb4_" << LowerName << "_get_" << PropertyLowerName << "list) int mb4_" << LowerName << "_get_" << PropertyLowerName << "(Mb4" << UpperName << " " << UpperName << ", char *str, int len);" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_STR_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "," << PropertyLowerName << ")" << std::endl;
+
+						Include << "/**" << std::endl;
+						Include << " * @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << "List" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << "List object" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return #Mb4" << PropertyUpperName << "List object" << std::endl;
+						Include << " */" << std::endl;
+						Include << "  Mb4" << PropertyUpperName << "List mb4_" << LowerName << "_get_" << PropertyLowerName << "list(Mb4" << UpperName << " " << UpperName << ");" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_OBJ_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "List," << PropertyLowerName << "list)" << std::endl;
+					}
+					else if ("ipi"==PropertyType)
+					{
+						Include << "/**" << std::endl;
+						Include << " * @deprecated @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << std::endl;
+						Include << " *" << std::endl;
+						Include << " * <b>This method is deprecated, please use #mb4_" << LowerName << "_get_" << PropertyLowerName << "list</b>" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << " object" << std::endl;
+						Include << " * @param str Returned string" << std::endl;
+						Include << " * @param len Number of characters available in return string" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return The number of characters in the string to copy (not including terminating NULL)" << std::endl;
+						Include << " */" << std::endl;
+						Include << "  LIBMB4_DEPRECATED(mb4_" << LowerName << "_get_" << PropertyLowerName << "list) int mb4_" << LowerName << "_get_" << PropertyLowerName << "(Mb4" << UpperName << " " << UpperName << ", char *str, int len);" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_STR_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "," << PropertyLowerName << ")" << std::endl;
+
+						Include << "/**" << std::endl;
+						Include << " * @see MusicBrainz4::C" << UpperName << "::" << PropertyUpperName << "List" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @param " << UpperName << " #Mb4" << UpperName << "List object" << std::endl;
+						Include << " *" << std::endl;
+						Include << " * @return #Mb4" << PropertyUpperName << "List object" << std::endl;
+						Include << " */" << std::endl;
+						Include << "  Mb4" << PropertyUpperName << "List mb4_" << LowerName << "_get_" << PropertyLowerName << "list(Mb4" << UpperName << " " << UpperName << ");" << std::endl;
+						Include << std::endl;
+
+						Source << "  MB4_C_OBJ_GETTER(" << UpperName << "," << LowerName << "," << PropertyUpperName << "List," << PropertyLowerName << "list)" << std::endl;
+					}
 					else
 					{
 						std::cerr << "Unexpected property type: '" << PropertyType << "'" << std::endl;
