@@ -10,7 +10,7 @@
    modify it under the terms of v2 of the GNU Lesser General Public
    License as published by the Free Software Foundation.
 
-   Flactag is distributed in the hope that it will be useful,
+   libmusicbrainz4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
@@ -18,7 +18,7 @@
    You should have received a copy of the GNU General Public License
    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
-     $Id: ctest.c 13385 2012-01-15 15:19:56Z adhawkins $
+     $Id$
 
 ----------------------------------------------------------------------------*/
 
@@ -28,11 +28,85 @@
 
 #include "musicbrainz4/mb4_c.h"
 
+void PrintRelationList(Mb4RelationList RelationList)
+{
+	char Type[256];
+	int count;
+
+	mb4_relation_list_get_targettype(RelationList,Type,sizeof(Type));
+	printf("Target type is '%s'\n",Type);
+
+	for (count=0;count<mb4_relation_list_size(RelationList);count++)
+	{
+		char Target[256];
+		char Type[256];
+
+		Mb4Relation Relation=mb4_relation_list_item(RelationList,count);
+
+		mb4_relation_get_target(Relation,Target,sizeof(Target));
+		mb4_relation_get_type(Relation,Type,sizeof(Type));
+
+		printf("Relation: %d - Type '%s', Target '%s'\n",count,Type,Target);
+	}
+}
+
 int main(int argc, const char *argv[])
 {
 	Mb4Query Query=0;
 	char DiscID[256]="tLGBAiCflG8ZI6lFcOt87vXjEcI-";
 
+	Query=mb4_query_new("ctest","test.musicbrainz.org",0);
+	if (Query)
+	{
+		char Version[256];
+		Mb4Metadata Metadata=0;
+
+		mb4_query_get_version(Query,Version,sizeof(Version));
+		printf("Version is '%s'\n",Version);
+
+		Metadata=mb4_query_query(Query,"work","b0d17375-5593-390e-a936-1a65ce74c630","",0,NULL,NULL);
+		if (Metadata)
+		{
+			Mb4Work Work=0;
+
+			Work=mb4_metadata_get_work(Metadata);
+			if (Work)
+			{
+				Mb4ISWCList ISWCList=0;
+				char ISWCStr[256];
+
+				mb4_work_get_iswc(Work,ISWCStr,sizeof(ISWCStr));
+				printf("ISWC: '%s'\n",ISWCStr);
+
+				ISWCList=mb4_work_get_iswclist(Work);
+				if (ISWCList)
+				{
+					int ISWCNum;
+
+					printf("Got ISWC list, size %d\n",mb4_iswc_list_size(ISWCList));
+
+					for (ISWCNum=0;ISWCNum<mb4_iswc_list_size(ISWCList);ISWCNum++)
+					{
+						Mb4ISWC ISWC=mb4_iswc_list_item(ISWCList,ISWCNum);
+
+						if (ISWC)
+						{
+							char ISWCStr[256];
+
+							mb4_iswc_get_iswc(ISWC,ISWCStr,sizeof(ISWCStr));
+
+							printf("Got ISWC %d - '%s'\n",ISWCNum,ISWCStr);
+						}
+					}
+				}
+			}
+		}
+
+		mb4_query_delete(Query);
+	}
+
+	return 0;
+
 	if (argc==2)
 	{
 		strncpy(DiscID,argv[1],sizeof(DiscID));
@@ -43,19 +117,72 @@ int main(int argc, const char *argv[])
 	if (Query)
 	{
 		Mb4Metadata Metadata=NULL;
+		char **ParamNames;
+		char **ParamValues;
 
 		printf("Got query\n");
 
 		mb4_query_set_username(Query,"username");
 		mb4_query_set_password(Query,"password");
 
+		ParamNames=malloc(sizeof(char *));
+		ParamNames[0]=malloc(256);
+		ParamValues=malloc(sizeof(char *));
+		ParamValues[0]=malloc(256);
+
+		strcpy(ParamNames[0],"inc");
+		strcpy(ParamValues[0],"artists release-groups url-rels work-level-rels work-rels artist-rels");
+
+		Metadata=mb4_query_query(Query,"recording","3631f569-520d-40ff-a1ee-076604723275","",1,ParamNames,ParamValues);
+		if (Metadata)
+		{
+			Mb4Recording Recording=mb4_metadata_get_recording(Metadata);
+			printf("Got metadata\n");
+
+			if (Recording)
+			{
+				Mb4RelationList RelationList=mb4_recording_get_relationlist(Recording);
+				Mb4RelationListList RelationListList=mb4_recording_get_relationlistlist(Recording);
+
+				printf("Got recording\n");
+
+				if (RelationListList)
+				{
+					int RelationListNum;
+
+					printf("Got relation list list, size %d\n",mb4_relationlist_list_size(RelationListList));
+
+					for (RelationListNum=0;RelationListNum<mb4_relationlist_list_size(RelationListList);RelationListNum++)
+					{
+						Mb4RelationList RelationList=mb4_relationlist_list_item(RelationListList,RelationListNum);
+
+						if (RelationList)
+						{
+							printf("Got relation list %d\n",RelationListNum);
+
+							PrintRelationList(RelationList);
+						}
+					}
+				}
+
+				if (RelationList)
+				{
+					printf("Got relation list\n");
+
+					PrintRelationList(RelationList);
+				}
+			}
+
+			mb4_metadata_delete(Metadata);
+		}
+
 		Metadata=mb4_query_query(Query,"collection",0,0,0,NULL,NULL);
 		if (Metadata)
 		{
 			printf("Got collections");
 			mb4_metadata_delete(Metadata);
-
 		}
+
 		Metadata=mb4_query_query(Query,"discid",DiscID,0,0,NULL,NULL);
 		if (Metadata)
 		{
@@ -94,8 +221,6 @@ int main(int argc, const char *argv[])
 
 						mb4_release_get_id(Release,ID,sizeof(ID));
 						printf("ID is '%s'\n",ID);
-
-						mb4_release_delete(Release);
 					}
 				}
 			}
@@ -128,8 +253,12 @@ void CompileTest()
 	Mb4DiscList DiscList=0;
 	Mb4FreeDBDisc FreeDBDisc=0;
 	Mb4FreeDBDiscList FreeDBDiscList=0;
+	Mb4IPI IPI=0;
+	Mb4IPIList IPIList=0;
 	Mb4ISRC ISRC=0;
 	Mb4ISRCList ISRCList=0;
+	Mb4ISWC ISWC=0;
+	Mb4ISWCList ISWCList=0;
 	Mb4Label Label=0;
 	Mb4LabelInfo LabelInfo=0;
 	Mb4LabelInfoList LabelInfoList=0;
@@ -151,10 +280,13 @@ void CompileTest()
 	Mb4RecordingList RecordingList=0;
 	Mb4Relation Relation=0;
 	Mb4RelationList RelationList=0;
+	Mb4RelationListList RelationListList=0;
 	Mb4Release Release=0;
 	Mb4ReleaseGroup ReleaseGroup=0;
 	Mb4ReleaseGroupList ReleaseGroupList=0;
 	Mb4ReleaseList ReleaseList=0;
+	Mb4SecondaryType SecondaryType=0;
+	Mb4SecondaryTypeList SecondaryTypeList=0;
 	Mb4Tag Tag=0;
 	Mb4TagList TagList=0;
 	Mb4TextRepresentation TextRepresentation=0;
@@ -182,6 +314,11 @@ void CompileTest()
 
 	mb4_alias_get_locale(Alias,Str,Size);
 	mb4_alias_get_text(Alias,Str,Size);
+	mb4_alias_get_sortname(Alias,Str,Size);
+	mb4_alias_get_type(Alias,Str,Size);
+	mb4_alias_get_primary(Alias,Str,Size);
+	mb4_alias_get_begindate(Alias,Str,Size);
+	mb4_alias_get_enddate(Alias,Str,Size);
 	Alias=mb4_alias_clone(Alias);
 	mb4_alias_delete(Alias);
 
@@ -199,6 +336,8 @@ void CompileTest()
 	mb4_artist_get_gender(Artist,Str,Size);
 	mb4_artist_get_country(Artist,Str,Size);
 	mb4_artist_get_disambiguation(Artist,Str,Size);
+	mb4_artist_get_ipi(Artist,Str,Size);
+	IPIList=mb4_artist_get_ipilist(Artist);
 	Lifespan=mb4_artist_get_lifespan(Artist);
 	AliasList=mb4_artist_get_aliaslist(Artist);
 	RecordingList=mb4_artist_get_recordinglist(Artist);
@@ -253,16 +392,26 @@ void CompileTest()
 	FreeDBDisc=mb4_freedbdisc_clone(FreeDBDisc);
 	mb4_freedbdisc_delete(FreeDBDisc);
 
+	mb4_ipi_get_ipi(IPI,Str,Size);
+	IPI=mb4_ipi_clone(IPI);
+	mb4_ipi_delete(IPI);
+
 	mb4_isrc_get_id(ISRC,Str,Size);
 	RecordingList=mb4_isrc_get_recordinglist(ISRC);
 	ISRC=mb4_isrc_clone(ISRC);
 	mb4_isrc_delete(ISRC);
 
+	mb4_iswc_get_iswc(ISWC,Str,Size);
+	ISWC=mb4_iswc_clone(ISWC);
+	mb4_iswc_delete(ISWC);
+
 	mb4_label_get_id(Label,Str,Size);
 	mb4_label_get_type(Label,Str,Size);
 	mb4_label_get_name(Label,Str,Size);
 	mb4_label_get_sortname(Label,Str,Size);
 	DummyInt=mb4_label_get_labelcode(Label);
+	mb4_label_get_ipi(Label,Str,Size);
+	IPIList=mb4_label_get_ipilist(Label);
 	mb4_label_get_disambiguation(Label,Str,Size);
 	mb4_label_get_country(Label,Str,Size);
 	Lifespan=mb4_label_get_lifespan(Label);
@@ -283,6 +432,7 @@ void CompileTest()
 
 	mb4_lifespan_get_begin(Lifespan,Str,Size);
 	mb4_lifespan_get_end(Lifespan,Str,Size);
+	mb4_lifespan_get_ended(Lifespan,Str,Size);
 	Lifespan=mb4_lifespan_clone(Lifespan);
 	mb4_lifespan_delete(Lifespan);
 
@@ -366,6 +516,7 @@ void CompileTest()
 	DummyResult=mb4_query_get_lastresult(Query);
 	DummyInt=mb4_query_get_lasthttpcode(Query);
 	mb4_query_get_lasterrormessage(Query,Str,Size);
+	mb4_query_get_version(Query,Str,Size);
 	mb4_query_delete(Query);
 
 	DummyInt=mb4_rating_get_votescount(Rating);
@@ -382,6 +533,7 @@ void CompileTest()
 	PUIDList=mb4_recording_get_puidlist(Recording);
 	ISRCList=mb4_recording_get_isrclist(Recording);
 	RelationList=mb4_recording_get_relationlist(Recording);
+	RelationListList=mb4_recording_get_relationlistlist(Recording);
 	TagList=mb4_recording_get_taglist(Recording);
 	UserTagList=mb4_recording_get_usertaglist(Recording);
 	Rating=mb4_recording_get_rating(Recording);
@@ -427,6 +579,7 @@ void CompileTest()
 
 	mb4_releasegroup_get_id(ReleaseGroup,Str,Size);
 	mb4_releasegroup_get_type(ReleaseGroup,Str,Size);
+	mb4_releasegroup_get_primarytype(ReleaseGroup,Str,Size);
 	mb4_releasegroup_get_firstreleasedate(ReleaseGroup,Str,Size);
 	mb4_releasegroup_get_title(ReleaseGroup,Str,Size);
 	mb4_releasegroup_get_disambiguation(ReleaseGroup,Str,Size);
@@ -437,6 +590,7 @@ void CompileTest()
 	UserTagList=mb4_releasegroup_get_usertaglist(ReleaseGroup);
 	Rating=mb4_releasegroup_get_rating(ReleaseGroup);
 	UserRating=mb4_releasegroup_get_userrating(ReleaseGroup);
+	SecondaryTypeList=mb4_releasegroup_get_secondarytypelist(ReleaseGroup);
 	ReleaseGroup=mb4_releasegroup_clone(ReleaseGroup);
 	mb4_releasegroup_delete(ReleaseGroup);
 
@@ -455,6 +609,7 @@ void CompileTest()
 	Recording=mb4_track_get_recording(Track);
 	DummyInt=mb4_track_get_length(Track);
 	ArtistCredit=mb4_track_get_artistcredit(Track);
+	mb4_track_get_number(Track,Str,Size);
 	Track=mb4_track_clone(Track);
 	mb4_track_delete(Track);
 
@@ -478,6 +633,7 @@ void CompileTest()
 	UserTagList=mb4_work_get_usertaglist(Work);
 	Rating=mb4_work_get_rating(Work);
 	UserRating=mb4_work_get_userrating(Work);
+	mb4_work_get_language(Work,Str,Size);
 
 	mb4_alias_list_size(AliasList);
 	Alias=mb4_alias_list_item(AliasList,0);
@@ -535,6 +691,13 @@ void CompileTest()
 	DummyInt=mb4_freedbdisc_list_get_offset(FreeDBDiscList);
 	mb4_freedbdisc_list_delete(FreeDBDiscList);
 
+	DummyInt=mb4_ipi_list_size(IPIList);
+	ISRC=mb4_ipi_list_item(IPIList,0);
+	ISRCList=mb4_ipi_list_clone(IPIList);
+	DummyInt=mb4_ipi_list_get_count(IPIList);
+	DummyInt=mb4_ipi_list_get_offset(IPIList);
+	mb4_ipi_list_delete(IPIList);
+
 	DummyInt=mb4_isrc_list_size(ISRCList);
 	ISRC=mb4_isrc_list_item(ISRCList,0);
 	ISRCList=mb4_isrc_list_clone(ISRCList);
@@ -542,6 +705,13 @@ void CompileTest()
 	DummyInt=mb4_isrc_list_get_offset(ISRCList);
 	mb4_isrc_list_delete(ISRCList);
 
+	DummyInt=mb4_iswc_list_size(ISWCList);
+	ISWC=mb4_iswc_list_item(ISWCList,0);
+	ISWCList=mb4_iswc_list_clone(ISWCList);
+	DummyInt=mb4_iswc_list_get_count(ISWCList);
+	DummyInt=mb4_iswc_list_get_offset(ISWCList);
+	mb4_iswc_list_delete(ISWCList);
+
 	DummyInt=mb4_label_list_size(LabelList);
 	Label=mb4_label_list_item(LabelList,0);
 	LabelList=mb4_label_list_clone(LabelList);
@@ -600,6 +770,13 @@ void CompileTest()
 	DummyInt=mb4_relation_list_get_offset(RelationList);
 	mb4_relation_list_delete(RelationList);
 
+	DummyInt=mb4_relationlist_list_size(RelationListList);
+	Relation=mb4_relationlist_list_item(RelationListList,0);
+	RelationList=mb4_relationlist_list_clone(RelationListList);
+	DummyInt=mb4_relationlist_list_get_count(RelationListList);
+	DummyInt=mb4_relationlist_list_get_offset(RelationListList);
+	mb4_relationlist_list_delete(RelationListList);
+
 	DummyInt=mb4_release_list_size(ReleaseList);
 	Release=mb4_release_list_item(ReleaseList,0);
 	ReleaseList=mb4_release_list_clone(ReleaseList);
@@ -614,6 +791,13 @@ void CompileTest()
 	DummyInt=mb4_releasegroup_list_get_offset(ReleaseGroupList);
 	mb4_releasegroup_list_delete(ReleaseGroupList);
 
+	DummyInt=mb4_secondarytype_list_size(SecondaryTypeList);
+	SecondaryType=mb4_secondarytype_list_item(SecondaryTypeList,0);
+	ReleaseGroupList=mb4_secondarytype_list_clone(SecondaryTypeList);
+	DummyInt=mb4_secondarytype_list_get_count(SecondaryTypeList);
+	DummyInt=mb4_secondarytype_list_get_offset(SecondaryTypeList);
+	mb4_secondarytype_list_delete(SecondaryTypeList);
+
 	DummyInt=mb4_tag_list_size(TagList);
 	Tag=mb4_tag_list_item(TagList,0);
 	TagList=mb4_tag_list_clone(TagList);
@@ -646,4 +830,5 @@ void CompileTest()
 	DummyDouble=DummyDouble;
 	DummyBool=DummyBool;
 	DummyResult=DummyResult;
+	SecondaryType=SecondaryType;
 }
